// z64.h: 2012-04-16 sieve of Eratosthenes for prime numbers, ver.3
// (c) Khashin S.I.  khash2@gmail.com

//typedef unsigned __int64 UINT64;// For Visual C++, change this definition for other compilers
//typedef          __int64  INT64;// For Visual C++, change this definition for other compilers
typedef unsigned long long UINT64;
typedef          long long  INT64;
//typedef unsigned UINT64;		// for debug only

class z64 {
private:
	static unsigned size;			// sieve array size
	static unsigned char *v;		// sieve array v[0..size-1]
	static UINT64 maxPrime;			// maximal prime number in sieve
	static UINT64 NPrimes;			// total number of primes (including 2)
	static unsigned *w;				// Pi(1024*i)

	// private functions
	static void SetComposite(UINT64 n);// set n composite
	static void   fillFinish();		// finish fill (maxPrime, NPrimes, v8, ...)

	// sieve of Eratosthenes private functions
	static bool   EisPrime( UINT64 n);	// is prime(n)?
	static UINT64 EithPrime( UINT64 i);	// i-th prime number,  Maple: ithprime(i+1)
	static UINT64 EPi( UINT64 n);		// the number of primes <= n, Pi(0)=Pi(1)=0, Pi(2)=1, Pi(3)=Pi(4)=2,...
	//
	static UINT64 factor_Pollard_rho (UINT64 n);// Pollard \rho -method of number factorization
	static bool   isPrime32(unsigned n);// full primality test till 2^32

public:
	// constructor and destructor do nothing
	z64 (){};
	~z64(){};

	static void    toString(char *s, UINT64 n, int w=0);			// s <- to String of width w
	static char   *toString(UINT64 n, int w=0);						// return to String of width w
	// 64-bit calculations:
	static bool	   isFullSq (UINT64 n);								// n is full square?
	static bool	   isDiv357_11_13(UINT64 n);						// n is div by 3,5,7,11,13?
	static UINT64  gcd      (UINT64 a, UINT64 b);					// GCD(a,b)
	static UINT64  mul_mod  (UINT64 a, UINT64 b, UINT64 n);			// (a*b)%c
	static UINT64  pow_mod  (UINT64 a, UINT64 b, UINT64 c);			// (a^b)%c
	static bool  MillerRabin(UINT64 a,UINT64 n);					// MR primality test for n with base a
	static void    euclid   (UINT64 a, UINT64 b, INT64 &x, INT64 &y, UINT64 &d); // find x,y,d: a*x+b*y=d=gcd(a,b)
	static UINT64  inverse  (UINT64 a, UINT64 n);					// 1/a mod n
	static bool    chineese (UINT64 a1, UINT64 b1, UINT64 a2, UINT64 b2, UINT64 &a3, UINT64 &b3); // cheneese theorem

	// sieve of Eratosthenes
	static void   Efill(UINT64 mN);		// fill sieve for primes till mN
	static void   Efree();				// free memory
	static int    Esave(const char *fname);	// return error code
	static int    Eload(const char *fname);	// return error code
	// if( z64::Eload("C:\\a\\primes\\sieve16M.dat") ) return; // till 2^24
	// if( z64::Eload("C:\\a\\primes\\sieve.dat"   ) ) return; // till 2^32
	// if( z64::Eload("C:\\a\\primes\\sieve16G.dat") ) return; // till 2^34

	//
	static UINT64 maxN();				// return maximal N
	static UINT64 nPrimes();			// total number of primes in the sieve (including 2)
	static UINT64 maxP();				// return maximal prime in the sieve

	// Primes
	static bool   isPrime( UINT64 n);	// is prime(n)?
	static UINT64 p( UINT64 i);			// i-th prime,  Maple: ithprime(i), p(0)=0, p(1)=2, p(2)=3,...
	static UINT64 Pi( UINT64 n);		// the number of primes <= n, Pi(0)=Pi(1)=0, Pi(2)=1, Pi(3)=Pi(4)=2,...
	static UINT64 NextPrime( UINT64 n); // prime p>n, NextPrime(3)=5, NextPrieme(4)=5, NextPrime(0)=NextPrime(1)=2, NextPrime(2)=3
	static UINT64 PrevPrime( UINT64 n); // prime p<n, PrevPrime(0)=PrevPrime(1)=PrevPrime(2)=0, PrevPrime(4)=3

	// Factoring
	// returns divisor of n (or 1, if not found)
	static UINT64 primeFactor( UINT64 n);		// prime divisor or 1, if not found

	// Rest
	static UINT64 order( UINT64 a, UINT64 n);	// order of a by modulo prime n
	static int    jacobi(UINT64 a, UINT64 n);	// Jacobi symbol
	static char * factorString(UINT64 n);		// string with factorisation: 180200 -> 2^3*5^2*17*53
	static UINT64 msqrt( UINT64 a, UINT64 p);	// sqrt(a) mod p, p must be prime!

};
//
